version 8           // tell Stata which version program was created in

// create or overwrite output file cie.smcl in ascii format

capture log close
log using cie_gph.log, replace                       
set matsize 10100
clear



*****************************************************************************************************
* 					    This Section does Monte Carlo sims of the T3 tax
*****************************************************************************************************

// Generate random emissions data from 1950 to 2120

	matrix e=J(170,1,0.25)				//	column of 0.25's
	matrix u=matuniform(170,1)				//	column of uniform random #s
	matrix u=u-e			
	matrix u=375*u						// 	scale up to annual change in emissions
	
	matrix e[1,1]=1630					// Now reuse e vector to make emissions series
									// Start at 1950=1630 to mimic CDIAC carbon series
	forvalues i=2/170 	{
		matrix e[`i',1]=e[`i'-1,1]+u[`i',1]	// fill e vector
					}
	svmat e
	rename e1 e					// convert matrix e to variable

	egen year=fill(1951/2120)				// generate time variable
	tsset year

	generate ebar=(e+L1.e+L2.e+L3.e+L4.e)/5	// generate ebar=ma(5) on e

	forvalues i=1/20		{
		generate e`i'=L`i'.e				// generate 20 lags of e for use in state variable
					}
	mvencode e1-e20, mv(2900)				// generate numeric n/a codes so matrix will multiply: rows will later be dropped
	mkmat e1-e20, matrix(emit20)			// put lags and cycle into 170x21 matrix emit




//  	INITIALIZE MONTE CARLO LOOP. COUNTER is `j'

//	j=5: 		DO RUN TO GENERATE 5 SAMPLE GRAPHS. EXIT AFTER GRAPHING COMPONENT. USE CIE_10K.LOG
//	j=10,000:	DO RUN TO GENERATE DISTRIBUTIONAL STATS. COMMENT OUT GRAPHING COMPONENT! USE CIE_GPH.LOG


		matrix beta=J(10000,1,1)		// initialize coefficient matrix etc. Change length to equal # repetitions
		matrix rsq=beta
		matrix taobeta=beta
		matrix cyclfreq=beta
		matrix cyclamp=beta
		matrix sbeta=beta
		matrix corrtt=beta
		matrix cooling=beta
		matrix over=beta
		matrix con=beta
		matrix t2000=beta
		matrix t2100=beta
		matrix gamma=beta
		matrix betaone=beta
		matrix outlier=beta

		forvalues j=1/5	{			// start the loop





//	Generate random state variable		


//		make random coefs, year variable and other vars
	matrix c20=matuniform(20,1)				// generates random weights u(0,1) for 20 lag terms
	matrix c21=matuniform(1,1)				// generate random coeff for cycle u(0,1)
	matrix c=J(20,1,1)					// initialize state coeff's vector 
	mkmat year, matrix(y)					// generates a vector with the year entries
	matrix e21=J(170,1,1) 				// initialize the 21st state var column, will become the cycle term
	matrix cycle=e21						// initialize the cycle vector
	matrix trend=e21						// initialize the trend vector


//		rescale random weights into declining 0-1 range and write to c
	forvalues i=1/20 	{
		qui	matrix c[`i',1]=(1/`i') * c20[`i',1]	
					}	


//		Make cycle term
	forvalues i=1/170	{
		qui	matrix cycle[`i',1] = cos((y[`i',1]-1950)*50*(1-c21[1,1])*3.1415927/180 )*c21[1,1]/2
								// generate cycle component with random freq and ampl
								// c21[1,1] is random parameter (0,1)
								// cycle term ranges +/- 0.5. 
								// small value of c21 implies low amplitude
								// low amplitude is paired with high frequency and vice versa
								// frequency adjustment is 50*(1-c21)
					}

//		Make trend term
	forvalues i=1/170	{
		qui 	matrix trend[`i',1] = (y[`i',1]-1950)*c20[2,1]/67
					}			// generate trend component with random size
								// It will take a negative sign
								// slope to range from -0 to -0.015 per year



//		Make core state variable: (170x20).(20x1) = 170x1 vector. 
	matrix statecor=emit20*c	



//		Convert matrices to variables
	svmat statecor					
		rename statecor1 statecor
	svmat cycle
		rename cycle1 cycle
	svmat trend
		rename trend1 trend




//		Make state variable
	qui	replace statecor=statecor/5000
	qui	generate state = statecor+cycle-trend		// add cycle+possibility of negative trend
	qui 	matrix t2000[`j',1]=state[50]			// record the year 2000 state variable value
	qui 	matrix t2100[`j',1]=state[150]			// record the year 2100 state variable value




//		Make the optimal tax term using dV/dt assuming D=10*s^2
	matrix cbeta=J(20,1,1)					// initialize coeffs for making value fn terms

	forvalues i=1/20 	{
		qui	matrix cbeta[`i',1]=10*((1/1.03)^(`i'-1))*c[`i',1]	// annual discount terms*c weight
					}

	forvalues i=0/19		{
		qui	generate st`i'=F`i'.state			// 20 leads of state var, t+0 to t+19
					}
	qui mvencode st0-st19, mv(99)				// assign numeric value to missing leads at end (will be dropped later)

	mkmat st0-st19, matrix(st)					// assemble state var leads into matrix st

	matrix tao=st*cbeta						// make tao matrix: optimal tax
	svmat tao								// convert tao matrix to variable
		rename tao1 tao




//		Make the tax approximation (equation 8)
	qui generate gamma=(year/year)*(tao[50]+tao[51]+tao[52]+tao[53]+tao[54]) / 		///
			(state[50]*e[50]/ebar[50] + state[51]*e[51]/ebar[51] + state[51]*e[51]/ebar[51] ///
		    	+state[52]*e[52]/ebar[52] + state[53]*e[53]/ebar[53])
										// Sets gamma to equate avg of tao and ttilde over 2000-2004
										// (year/year) is just a way to force Stata to build a vector of ones 

	qui generate ttilde=gamma*e*state/ebar			// ttilde is a variable
	matrix gamma[`j',1]=gamma[1]				// gamma matrix entry is taken from gamma-variable, 1st position




//	Obtain correlation between tao and ttilde
	qui matrix accum R=tao ttilde , deviations noconstant
	qui matrix R=corr(R)
	qui matrix corrtt[`j',1]=R[2,1]


//	Diagnose the fit

	qui regress state year if year>1974 & year<2101
		matrix sbeta[`j',1]=_b[year]

	qui regress tao ttilde if year>1974 & year<2101
		matrix con[`j',1]=_b[_cons]
		matrix beta[`j',1]=_b[ttilde]		// beta between the 2
		matrix rsq[`j',1] =e(r2)			// r2 between the 2	

	qui regress tao year if year>1974 & year<2101
		matrix taobeta[`j',1]=_b[year]		// how rapidly tao increases
		matrix cyclamp[`j',1] =c21[1,1]/2		// cycle parameters
		matrix cyclfreq[`j',1]=10*(1-c21[1,1])		// 




//	GRAPHING COMPONENT. COMMENT THIS OUT FOR j=10,000 RUN (UNLESS YOU WANT 10,000 GRAPHS)

	line state year if year>1974 & year<2101, ylabel(0(1)4) xlabel(1975(25)2100) xtitle("")
		graph save state`j', replace 
	scatter tao ttilde year if year>1974 & year<2101, ///
			ylabel(0(20)80) xlabel(1975(25)2100) leg(off) msymbol(i i) c(l l) clc(gs0 gs9) clw(thin thick) xtitle("") 
		graph save tt`j'.gph  , replace



//		END OF MAIN LOOP
			drop state statecor tao ttilde gamma
			drop st0-st19
			drop trend cycle
				}


//	COMBINE GRAPHS. COMMENT OUT FOR j=10,000 RUNS

	graph combine 	state1.gph state2.gph state3.gph state4.gph state5.gph ///
				tt1.gph tt2.gph tt3.gph tt4.gph tt5.gph , colf cols(2) 
	graph save panel5, 		replace
	graph export panel5.eps, 	replace
exit






//		Generate report on tax approximation
svmat beta							//
	rename beta1 beta				// Convert matrices to variables for operations
svmat rsq							//
	rename rsq1 rsq
svmat taobeta
	rename taobeta1 taobeta
svmat sbeta
	rename sbeta1 sbeta
svmat cyclfreq
	rename cyclfreq1 cyclfreq
svmat cyclamp
	rename cyclamp1 cyclamp
svmat corrtt
	rename corrtt1 corrtt
svmat cooling
	rename cooling1 cooling
svmat over
	rename over1 over
svmat con
	rename con1 con
svmat t2000 
	rename t20001 t2000
svmat t2100 
	rename t21001 t2100
svmat gamma
	rename gamma1 gamma
svmat betaone
	rename betaone1 betaone
svmat outlier
	rename outlier1 outlier

replace betaone=(beta>0.98)*(beta<1.02)	// beta nearly 1.0
replace outlier=(beta<0)+(beta>3)		// beta is way off
replace cooling=(sbeta<0)				// did state rise or fall?
replace over=(beta>1)					// did ttilde overshoot tao?
replace cyclamp=1/cyclamp				// cyclamp prm is reciprocal
generate posbeta=(beta>0)				// correct sign

list beta con corrtt gamma cyclamp sbeta rsq t2100 in 1/100

sum beta con t2000 t2100 posbeta rsq corrtt sbeta taobeta gamma cyclfreq cyclamp cooling betaone outlier over 
sum beta con t2000 t2100 posbeta rsq corrtt sbeta taobeta gamma cyclfreq cyclamp cooling betaone over if outlier==1
sum beta con t2000 t2100 posbeta rsq corrtt sbeta taobeta gamma cyclfreq cyclamp cooling betaone over if outlier==0
sum beta con t2000 t2100 posbeta rsq corrtt sbeta taobeta gamma cyclfreq cyclamp cooling betaone over if betaone==1

drop if outlier==1					// get rid of outlier values

regress beta sbeta cyclfreq t2100 gamma
regress corrtt sbeta cyclfreq t2100 gamma

scatter corrtt gamma
	graph save corrtt_gamma, replace

histogram beta, normal normopts(clp(dash)) bin(80) kdensity legend() start(0)
	graph save beta, replace
	graph export beta.eps ,replace











*****************************************************************************************************
* 					    This Section computes the tax in equation (8) using real data
*****************************************************************************************************



*****************************************************************************************************
*                                       GET UAH DATA FOR TROPICS
*****************************************************************************************************

// Obtain data from UAH (T2 series)
clear
insheet uah1 using http://vortex.nsstc.uah.edu/data/msu/t2/uahncdc.mt
	split uah1
	keep uah11-uah16 uah19 uah112
	rename uah11 year
	rename uah12 month
	rename uah112 uah
	drop if _n == 1 | year=="1978" | year=="Year" | year=="Trend" 
	destring, replace

tsset year month
	generate mon = ym(year,month)
	format mon %tm

keep year month uah
drop if year>2009

sum
collapse (mean) uah, by(year)
	sum
	list year uah

*                    Write Data set
capture save "uah.dta", replace

*****************************************************************************************************
*                                       GET RSS DATA FOR TROPICS
*****************************************************************************************************

// Obtain data from RSS (TMT series)
clear
insheet rss1 using ///
http://www.remss.com/data/msu/monthly_time_series/RSS_Monthly_MSU_AMSU_Channel_TMT_Anomalies_Land_and_Ocean_v03_2.txt
	split rss1
	keep rss11-rss14 rss110-rss111
	rename rss11 year
	rename rss12 month
	rename rss14 rss
	drop if _n == 1 | _n == 2 | _n == 3 
	destring, replace

tsset year month
	generate mon = ym(year,month)
	format mon %tm

keep year month rss
drop if year>2009

sum
collapse (mean) rss, by(year)
	sum
	list year rss

*                    Write Data set
capture save "rss.dta", replace

*****************************************************************************************************
*                                       GET EMISSIONS DATA FROM CDIAC
*****************************************************************************************************
clear

insheet using "global.1751_2007.csv" //	get CDIAC data
	drop if year<1930
	
	keep year totalcarb*
	rename totalcarbo~u et

tsset year 
	tssmooth ma ebar=	et, window(20 1)		// generate ebar using k=20

drop if year<1979
generate eoe=et/ebar					// generate et over ebar

capture save "co2.dta", replace

*****************************************************************************************************
*                                       MERGE DATA SETS
*****************************************************************************************************
clear
	use "uah.dta"
	sort year 

	merge using "rss.dta"
	drop _merge
	sort year 

capture save "msu.dta", replace

clear
	use "msu.dta"
	merge using "co2.dta"

//		Use 3rd order polynomial regression to predict eoe 3 years fwd
generate year2=year^2/1000
generate year3=year*year2/1000
regress eoe year year2 year3
	predict eoe1 ,xb
	replace eoe=eoe1 if year>2006

summarize

*****************************************************************************************************
*                                       STATE-CONTINGENT TAX RATE
*****************************************************************************************************

*		Calibrate so 2002 = $15/tonne

generate statevar=(rss+uah)/2
generate tax1=eoe*statevar
	generate gamma=15/tax1[24]
	list gamma in 1/2
	generate tax=gamma*eoe*statevar

generate avgtax=(tax+tax[_n-1]+tax[_n-2]+tax[_n-3])/4

format tax %5.2f 
format avgtax %5.2f 
format statevar %6.3f 
format eoe %5.3f
list year tax avgtax statevar eoe , divider table 

scatter tax year, ytitle(Emissions tax $/tonne) xtitle(Year) yline(0) xmtick(##10) connect(l) ///
|| line avgtax year, clwidth(thick) 
	graph save   cie_tax.gph ,replace
	graph export cie_tax.eps ,replace

regress tax year
exit


